/* Copyright 2020-2022, Michele Santullo
 * This file is part of wrenpp.
 *
 * Wrenpp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wrenpp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with wrenpp.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <cstdint>

namespace wren {
	//from:
	//https://codereview.stackexchange.com/questions/48594/unique-type-id-no-rtti
	class TypeID {
		using sig = TypeID();

		sig* m_id;
		constexpr TypeID(sig* id) : m_id{id} {}

	public:
		template<typename T>
		friend constexpr TypeID type_id();

		TypeID (const TypeID&) = default;
		TypeID (TypeID&&) = default;
		~TypeID() noexcept = default;
		TypeID& operator= (const TypeID&) = default;
		TypeID& operator= (TypeID&&) = default;

		constexpr bool operator==(TypeID other) const noexcept;
		constexpr bool operator!=(TypeID other) const noexcept;
		constexpr bool operator<(TypeID other) const noexcept;
		constexpr std::uintptr_t id() const noexcept;
	};

	template<typename T>
	constexpr TypeID type_id() { return &type_id<T>; }

	constexpr bool TypeID::operator==(TypeID other) const noexcept {
		return id() == other.id();
	}

	constexpr bool TypeID::operator!=(TypeID other) const noexcept {
		return id() != other.id();
	}

	constexpr bool TypeID::operator<(TypeID other) const noexcept {
		return id() < other.id();
	}

	constexpr std::uintptr_t TypeID::id() const noexcept {
		return reinterpret_cast<std::uintptr_t>(m_id);
	}
} //namespace wren
