/* Copyright 2020-2022, Michele Santullo
 * This file is part of wrenpp.
 *
 * Wrenpp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wrenpp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with wrenpp.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "module_and_name.hpp"
#include "type_id.hpp"
#include "crc32.hpp"
#include <utility>
#include <string_view>

namespace wren {
	class VM;

	namespace detail {
		//Just an utility function to call a method on the class manager in vm
		//and return its return value without including the header for the
		//class manager. Useful in inline functions in header files where
		//including class_manager.hpp would propagate it everywhere.
		ModuleAndName fetch_class_name_from_manager (
			const VM& vm,
			TypeID hash
		);

		template <typename T>
		inline ModuleAndName wren_class_name_from_type (const VM& vm) {
			return fetch_class_name_from_manager(vm, type_id<T>());
		}
	} //namespace detail
} //namespace wren
